// random number generation functions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "RandomNumber.h"

#include <ctime>
#include <vector>
#include <iostream>
#include <gsl/gsl_randist.h>

using namespace ECSB;

// constructor
RNG::RNG(unsigned long s)
{
	FILE *fp;
	unsigned long newseed;

	// initialise uniform random number generator
	rng = gsl_rng_alloc(gsl_rng_mt19937);

	// seed generator randomly if seed set to zero
	seed = s;
	if(!seed) {
		if((fp = fopen("/dev/urandom", "r"))) {
			// set seed from /dev/urandom
			fread(&seed, sizeof(seed), 1, fp);
			fclose(fp);
		} else
			// set seed from time
			seed = (unsigned long) std::time(0);
	}
	gsl_rng_set(rng, seed);
}

// destructor
RNG::~RNG()
{
	gsl_rng_free(rng);
}

// get generator seed
unsigned long RNG::getSeed(void)
{
	return seed;
}

// generate a uniform random number on the interval [0,1)
double RNG::randomNumber(void)
{
	return gsl_rng_uniform(rng);
}

// generate a uniform random number on the interval (0,1)
double RNG::randomNumberPos(void)
{
	return gsl_rng_uniform_pos(rng);
}

// generate a random non-negative integer
unsigned long RNG::randomNonnegativeInteger(void)
{
	return gsl_rng_get(rng);
}

// generate a uniform random non-negative integer
unsigned long RNG::randomNonnegativeInteger(unsigned long range)
{
	return gsl_rng_uniform_int(rng, range);
}

// generate a gaussian random number
double RNG::gaussianRandomNumber(double mean, double sd)
{
	return (mean + gsl_ran_gaussian(rng, sd));
}

// generate a non-negative gaussian random number
double RNG::gaussianRandomNumberNN(double mean, double sd)
{
	register double r;

	r = mean + gsl_ran_gaussian(rng, sd);
	return ((r < 0.0) ? 0.0 : r );
}

// divide integer by two and round up or down randomly
unsigned long RNG::randomHalf(unsigned long val)
{
	if(!(val % 2))
		return (val / 2);
	return ((val / 2) + gsl_rng_uniform_int(rng, 2));
}

// flip a coin and returns true or false
bool RNG::flipCoin()
{
	if(gsl_rng_uniform_int(rng, 2))
		return true;
	else
		return false;
}

// generates a random subset from a larger set {min, min+1, ..., max-1, max}
std::list<long> RNG::randomSubset(long min, long max, unsigned long subset_size)
{
	std::vector<long> set;
	std::list<long> subset;
	unsigned long idx, j;
	long i;

	if(subset_size > (max - min + 1)) {
		std::cerr << "error: random subset cannot be larger than the set\n";
		exit(EXIT_FAILURE);
	}

	// initialize set from min to max
	for(i = min; i <= max; i++)
		set.push_back(i);

	// pick one element at a time from set and add it to subset
	for(j = 0; j < subset_size; j++) {
		idx = randomNonnegativeInteger(set.size());
		subset.push_back(set[idx]);
		set.erase(set.begin() + idx);
	}

	return subset;
}
